# -*- coding: utf-8 -*-
# Coding: BY MOHAMED_OS


from Plugins.Extensions.IPTVPlayer.compat import urllib_quote_plus
from Plugins.Extensions.IPTVPlayer.components.ihost import (CBaseHostClass,
                                                            CHostBase)
from Plugins.Extensions.IPTVPlayer.components.iptvplayerinit import \
    TranslateTXT as _
from Plugins.Extensions.IPTVPlayer.libs.e2ijson import loads as json_loads
from Plugins.Extensions.IPTVPlayer.tools.iptvtools import printDBG, printExc
from Plugins.Extensions.IPTVPlayer.tools.manipulateStrings import iterDictItems


def GetConfigList():
    optionList = []
    return optionList


def gettytul():
    return 'WatanFlix'


class WatanFlix(CBaseHostClass):

    def __init__(self):
        CBaseHostClass.__init__(self, {'history': 'watanflix', 'cookie': 'watanflix.cookie'})

        self.MAIN_URL = 'https://watanflix.com/'
        self.DEFAULT_ICON_URL = 'https://i.ibb.co/tsFN2d5/watanflix.png'

        self.HEADER = self.cm.getDefaultHeader()
        self.AJAX_HEADER = self.HEADER
        self.AJAX_HEADER.update({'X-Requested-With': 'XMLHttpRequest'})
        self.defaultParams = {'header': self.HEADER, 'use_cookie': True, 'load_cookie': True, 'save_cookie': True, 'cookiefile': self.COOKIE_FILE}

    def getPage(self, baseUrl, addParams={}, post_data=None):
        if addParams is {}:
            addParams = dict(self.defaultParams)

        sts, data = self.cm.getPage(self.cm.ph.stdUrl(baseUrl), addParams, post_data)
        return sts, data

    def listMainMenu(self, cItem):
        printDBG("WatanFlix.listMainMenu")
        MAIN_CAT_TAB = [
            {'category': 'movei', 'title': _('الأفـــلام'), 'icon': self.DEFAULT_ICON_URL, 'url': self.getFullUrl('/ar/category/الأفلام')},
            {'category': 'serie', 'title': _('مســلـســلات'), 'icon': self.DEFAULT_ICON_URL, 'url': self.getFullUrl('/ar/category/مسلسلات')},
            {'category': 'tvshow', 'title': _('بـــرامـــج'), 'icon': self.DEFAULT_ICON_URL, 'url': self.getFullUrl('/ar/category/برامج')},
            {'category': 'msrh', 'title': _('مسـرحـيـات'), 'icon': self.DEFAULT_ICON_URL, 'url': self.getFullUrl('/ar/category/مسرحيات')},
            {'category': 'search', 'title': _('Search'), 'search_item': True},
            {'category': 'search_history', 'title': _('Search history'), }]
        self.listsTab(MAIN_CAT_TAB, cItem)

    def listItems(self, cItem):
        printDBG("WatanFlix.listItems cItem[%s]" % (cItem))
        page = cItem.get('page', 1)

        sts, data = self.getPage(cItem['url'])
        if not sts:
            return

        nextPage = self.cm.ph.getDataBeetwenMarkers(data, ('<div', '>', 'paginate'), ('</div', '>'), True)[1]
        nextPage = self.getFullUrl(self.cm.ph.getSearchGroups(nextPage, '''href=['"]([^'^"]+?)['"][^>]*?>%s<''' % (page + 1))[0])

        tmp = self.cm.ph.getDataBeetwenMarkers(data, ('<div', '>', 'grid-list'), ('<div', '>', 'mainFooter'), True)[1]
        tmp = self.cm.ph.getAllItemsBeetwenMarkers(tmp, ('<div', '>', 'video-grid1'), ('</a>', '</div>'))
        for item in tmp:
            icon = self.getFullIconUrl(self.cm.ph.getSearchGroups(item, '''src=['"]([^"^']+?)['"][$>]''')[0])
            url = self.getFullUrl(self.cm.ph.getSearchGroups(item, '''href=['"]([^"^']+?)['"]''')[0])
            title = self.cleanHtmlStr(self.cm.ph.getDataBeetwenNodes(item, ('<h4', '>'), ('</span', '>'), False)[1])

            info = self.ph.std_title(title, with_ep=True)
            if title is not '':
                title = info.get('title_display')
            desc = info.get('desc')

            params = dict(cItem)
            params.update({'category': 'listItems', 'good_for_fav': True, 'EPG': True, 'title': title, 'url': url, 'icon': self.cm.ph.stdUrl(icon), 'desc': desc})
            self.addDir(params)

        if nextPage is not '':
            params = dict(cItem)
            params.update({'title': _("Next page"), 'url': nextPage, 'page': page + 1})
            self.addDir(params)

    def exploreItems(self, cItem):
        printDBG("WatanFlix.exploreItems cItem[%s]" % (cItem))

        sts, data = self.getPage(cItem['url'])
        if not sts:
            return

        descObj = self.getArticleContent(cItem, data)[0]
        desc = []
        for t in ['year', 'genre', 'desc']:
            if t in descObj['other_info']:
                desc.append(descObj['other_info'][t])
        desc = ' | '.join(desc) + '[/br]' + descObj['text']

        tmp = self.cm.ph.getDataBeetwenMarkers(data, ('<div', '>', 'slidingSeries'), ('<div', '>', 'clearfix'), True)[1]
        tmp = self.cm.ph.getAllItemsBeetwenMarkers(tmp, ('<div', '>', 'item'), ('</div>', '</div>'))
        for item in tmp:
            icon = self.getFullIconUrl(self.cm.ph.getSearchGroups(item, '''src=['"]([^"^']+?)['"]''')[0])
            url = self.getFullUrl(self.cm.ph.getSearchGroups(item, '''href=['"]([^"^']+?)['"]''')[0])
            title = self.cleanHtmlStr(self.cm.ph.getDataBeetwenNodes(item, ('<p', '>'), ('</p', '>'), False)[1])

            info = self.ph.std_title(title, with_ep=True)
            if title is not '':
                title = info.get('title_display')
            otherInfo = f"{info.get('desc')}\n{desc}"

            params = dict(cItem)
            params.update({'good_for_fav': True, 'EPG': True, 'title': title, 'url': url, 'icon': icon, 'desc': otherInfo})
            self.addVideo(params)

    def showSearch(self, cItem):
        printDBG("WatanFlix.exploreItems cItem[%s]" % (cItem))

        sts, data = self.getPage(cItem['url'])
        if not sts:
            return

        stsObj = json_loads(self.ph.decodeHtml(data))
        for key, section in iterDictItems(stsObj):
            for item in section:
                params = dict(cItem)
                params.update({'good_for_fav': True, 'EPG': True, 'title': item['title'], 'url': self.getFullUrl(item['url'])})
                self.addDir(params)

    def listSearchResult(self, cItem, searchPattern, searchType):
        printDBG("WatanFlix.listSearchResult cItem[%s], searchPattern[%s] searchType[%s]" % (cItem, searchPattern, searchType))
        url = self.getFullUrl('/ar/search?q={}'.format(urllib_quote_plus(searchPattern)))
        params = {'name': 'category', 'category': 'list_items', 'good_for_fav': False, 'url': url}
        self.showSearch(params)

    def getLinksForVideo(self, cItem):
        printDBG("WatanFlix.getLinksForVideo [%s]" % cItem)

        url = cItem.get('url', '')
        if 1 is self.up.checkHostSupport(url):
            return self.up.getVideoLinkExt(url)

    def getVideoLinks(self, videoUrl):
        printDBG("WatanFlix.getVideoLinks [%s]" % videoUrl)

        if self.cm.isValidUrl(videoUrl):
            return self.up.getVideoLinkExt(videoUrl)

    def getArticleContent(self, cItem, data=None):
        printDBG("WatanFlix.getArticleContent [%s]" % cItem)
        otherInfo = {}

        sts, data = self.getPage(cItem['url'])
        if not sts:
            return

        tmp = self.cm.ph.getDataBeetwenMarkers(data, ('<div', '>', 'series_info'), ('<div', '>', 'clearfix'), True)[1]

        Info = self.cleanHtmlStr(self.cm.ph.getDataBeetwenNodes(tmp, ('<div', '>', 'year'), ('</div', '>'), False)[1])
        if Info is not '':
            otherInfo['year'] = Info

        Info = self.cleanHtmlStr(self.cm.ph.getDataBeetwenNodes(tmp, ('<div', '>', 'genre'), ('</div', '>'), False)[1])
        if Info is not '':
            otherInfo['genre'] = Info

        desc = self.cleanHtmlStr(self.cm.ph.getDataBeetwenNodes(tmp, ('<div', '>', 'description'), ('</p', '>'), False)[1])
        if desc is '':
            desc = cItem['desc']

        icon = self.getFullIconUrl(self.cm.ph.getSearchGroups(tmp, '''src=['"]([^"^']+?)['"]''')[0])
        if icon is '':
            icon = cItem['icon']

        return [{'title': cItem['title'], 'text': desc, 'images': [{'title': '', 'url': icon}], 'other_info': otherInfo}]

    def handleService(self, index, refresh=0, searchPattern='', searchType=''):
        printDBG('handleService start')

        CBaseHostClass.handleService(self, index, refresh, searchPattern, searchType)

        name = self.currItem.get("name", '')
        category = self.currItem.get("category", '')
        printDBG(f"handleService: |||||||||||||||||||||||||||||||||||| name[{name}], category[{category}] ")
        self.currList = []

    # MAIN MENU
        if name is None and category is '':
            self.listMainMenu({'name': 'category', 'type': 'category'})
        elif category is 'movei' or category is 'serie' or category is 'tvshow' or category is 'msrh':
            self.listItems(self.currItem)
        elif category is 'listItems' or category is 'list_items':
            self.exploreItems(self.currItem)
    # SEARCH
        elif category in ["search", "search_next_page"]:
            cItem = dict(self.currItem)
            cItem.update({'search_item': False, 'name': 'category'})
            self.listSearchResult(cItem, searchPattern, searchType)
    # HISTORIA SEARCH
        elif category is "search_history":
            self.listsHistory({'name': 'history', 'category': 'search'}, 'desc', _("Type: "))
        else:
            printExc()

        CBaseHostClass.endHandleService(self, index, refresh)


class IPTVHost(CHostBase):

    def __init__(self):
        CHostBase.__init__(self, WatanFlix(), True, [])

    def withArticleContent(self, cItem):
        if cItem['type'] is not 'video' and cItem['category'] is not 'exploreItems':
            return False
        return True
